/*
 * File:        Lab3_am859_jzw8.c
 * Author:      Anthony McNicoll, Jonathan Wu
 * Target PIC:  PIC32MX250F128B
 */

#define _SUPPRESS_PLIB_WARNING 1
#include "plib.h"           // PLIB
#include "config.h"         // Configuration for clocks & protothreads
#include "param.h"          // Parameters, GUI dimensions and colors, etc.
#include "tft_master.h"     // Graphics libraries
#include "tft_gfx.h"
#include "pt_cornell.h"     // Threading library
#include "tool_macros.h"    // Macros (pullups, fixed point, etc)
#include <xc.h>             // Interrupt libraries
#include <sys/attribs.h>

// Thread Structures
// Note that measurements happen in interrupts.
// "Handler" is a thread that handles button events like clear, null.
static struct pt pt_report, pt_serial, pt_adc;

// Measurement globals
// Per-cycle measurements:
volatile float rpm          = 0;
volatile float p_rpm_x      = 0;
volatile float p_rpm_y      = 0;
volatile float amp_x        = 0;
volatile float amp_y        = 0;
volatile int   amp_meas_x   = 0;
volatile int   amp_meas_y   = 0;
volatile float p_amp_x      = 0;
volatile float p_amp_y      = 0;
volatile float phs_x        = 0;
volatile float phs_y        = 0;
volatile float phs_meas_x   = 0;
volatile float phs_meas_y   = 0;
volatile float p_phs_x      = 0;
volatile float p_phs_y      = 0;

// Conversion scale, times 10
volatile unsigned short s_x = 70;
volatile unsigned short s_y = 70;

// Global state variables
volatile unsigned short x_null   = 0;
volatile unsigned short y_null   = 0;
volatile unsigned short start_x_null = 0;
volatile unsigned short start_y_null = 0;
volatile unsigned short getting_x_null = 0;
volatile unsigned short getting_y_null = 0;
volatile unsigned short accepting_press = 1;      // Cheesy debounce

// Null vector array. This is recorded at up to MEASURE_F/2 when the shaft
// is at MIN_RPM. Example for 8 KHz, 3.33 Hz we could get 2400 12-bit samples.
// Accompany with a fixed-point index a-la DDS.
// Let's do 720 index so we can double phase float for index. YOLO.
volatile int null_arr_x[720];
volatile int null_arr_y[720];

// String buffer
char buffer[60];

// Subsystems: these are "copy and paste" code, separate files for clarity.
#include "adc.h"
#include "per_sample.h"
#include "per_cycle.h"
#include "gui.h"
#include "buttons.h"

// === Serial Communication Thread ===
// Simply send current readings over serial at some time interval.
static PT_THREAD (protothread_serial(struct pt *pt))
{
    PT_BEGIN(pt);
    while (1){
        PT_YIELD_TIME_msec(SER_T)
        printf("%.0f %.0f %.0f %.0f %.0f\r\n", rpm, amp_x, phs_x, amp_y, phs_y);
    }
    PT_END(pt);
}

// === Main  ===================================================================
void main(void) {
  // Optimizes system performance for out peripheral bus clock
  SYSTEMConfigPerformance(PBCLK);
  
  // Obsessively turn analog input, comparators off
  ANSELA = 0; ANSELB = 0; CM1CON = 0; CM2CON = 0;

  // Sets up protothreads with UART at 115200
  PT_setup();
  
  // Initialize the display
  init_gui();
  
  // Initialize all the modules
  initCycleCapture();
  initWaveSampler();
  initADC();
  initButtons();

  // This means ISR's are assigned based on vectors - duh.
  INTEnableSystemMultiVectoredInt();
  
  // Initialize the threads
  PT_INIT(&pt_report);
  PT_INIT(&pt_adc);
  PT_INIT(&pt_serial);
  
  // round-robin scheduler for threads
  while (1){
      PT_SCHEDULE(protothread_report(&pt_report));
      PT_SCHEDULE(protothread_serial(&pt_serial));
      PT_SCHEDULE(protothread_adc(&pt_adc));
  }
}